using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.IO;
using System.Xml;

namespace ImageResizer.util
{
	/// <summary>
	/// Zusammenfassung fr AppSetting.
	/// </summary>
	public class AppSettings : IDisposable
	{
		// Fields
		private const string APPSETTINGS_ELEMENT = "configuration//appSettings";
		private XmlDocument _cfg;
		private bool mblnAutoSave;
		private bool _blnDirty;
		private bool _blnDisposed;
		private bool _blnDisposing;
		private string _strFileName;
		private const string NEWELEMENT = "add";
		private XmlNode _xAS;
		private const string XPATH_KEY_ADD = "//add";
		private const string XPATH_KEY_ADD_KEY = "//add[@key='{0}']";


		//Properties
		public bool AutoSave
		{
			get
			{
				return this.mblnAutoSave;
			}
			set
			{
				this.mblnAutoSave = value;
			}
		}
	
		public bool Dirty
		{
			get
			{
				return this._blnDirty;
			}
		}
 


		//Constructors
		public AppSettings(string ConfigFile): this(ConfigFile, false)
		{
		}
		
		public AppSettings(string ConfigFile, bool AutoSave)
		{
			this._cfg = new XmlDocument();
			if (ConfigFile.Length == 0)
			{
				throw new ArgumentNullException("You must pass in a valid file name.");
			}
			if (!File.Exists(ConfigFile))
			{
				throw new FileNotFoundException(string.Format("The file name specified {0} does not exist.", ConfigFile));
			}
			try
			{
				this._cfg.Load(ConfigFile);
			}
			catch (Exception exception2)
			{
				//ProjectData.SetProjectError(exception2);
				Exception exception1 = exception2;
				throw new FileLoadException("The file name specified could not be loaded. Please see InnerException for more information", exception1);
				//ProjectData.ClearProjectError();
			}
			this._xAS = this._cfg.SelectSingleNode("configuration//appSettings");
			if (this._xAS == null)
			{
				throw new ConfigurationException("The file specified, while a valid XML document, is not a valid configuration file.");
			}
			this._strFileName = ConfigFile;
			this.AutoSave = AutoSave;
		}
 

		//Methods
		public void Add(ref AppSetting NewSetting)
		{
			NewSetting = this.Add(NewSetting.Key, NewSetting.Value);
		}
 
		public AppSetting Add(string Key, string Value)
		{
			XmlElement element1 = this._cfg.CreateElement("add");
			XmlAttribute attribute1 = this._cfg.CreateAttribute("key");
			attribute1.Value = Key;
			element1.Attributes.Append(attribute1);
			attribute1 = this._cfg.CreateAttribute("value");
			attribute1.Value = Value;
			element1.Attributes.Append(attribute1);
			this._xAS.AppendChild(element1);
			this._blnDirty = true;
			if (this.AutoSave)
			{
				this.Save();
			}
			return new AppSetting(Key, Value, this);
		}
 

		public void Dispose()
		{
			this._blnDisposing = true;
			if (this.Dirty)
			{
				this.Save();
			}
			this._blnDisposed = true;
			this._blnDisposing = false;
			GC.SuppressFinalize(this);
		}
 
//		protected override void Finalize()
//		{
//			base.Finalize();
//			if (this.Dirty && (!this.mblnDisposed && !this.mblnDisposing))
//			{
//				this.Dispose();
//			}
//		}
	
		public AppSetting[] GetAllItems()
		{
			IEnumerator enumerator1;
			XmlNodeList list1 = this._xAS.SelectNodes("//add");
			AppSetting[] settingArray1 = new AppSetting[(list1.Count - 1) + 1];
			int num1 = -1;

			enumerator1 = list1.GetEnumerator();
			try
			{
				
				while (enumerator1.MoveNext())
				{
					XmlNode node1 = (XmlNode) enumerator1.Current;
					num1++;
					XmlAttributeCollection collection2 = node1.Attributes;
					settingArray1[num1] = new AppSetting(collection2.Item(0).Value, collection2.Item(1).Value, this);
					collection2 = null;
				}
			}
			finally
			{
				if (enumerator1 is IDisposable)
				{
					((IDisposable) enumerator1).Dispose();
				}
			}
			return settingArray1;
		}
 
		public AppSetting Item(string Key)
		{
			string text1 = "//add[@key='{0}']";
			XmlNode node1 = this._xAS.SelectSingleNode(string.Format(text1, Key));
			if (node1 == null)
			{
				throw new ArgumentOutOfRangeException("Key", Key, "The item you wanted to update does not exists.");
			}
			return new AppSetting(Key, node1.Attributes.Item(1).Value, this);
		}
 
		public void RemoveByKey(AppSetting Setting)
		{
			this.RemoveByKey(Setting.Key);
		}
 
		public void RemoveByKey(string Key)
		{
			string text1 = "//add[@key='{0}']";
			XmlNode node1 = this._xAS.SelectSingleNode(string.Format(text1, Key));
			if (node1 == null)
			{
				node1.ParentNode.RemoveChild(node1);
			}
			this._blnDirty = true;
			if (this.AutoSave)
			{
				this.Save();
			}
		}
 
		public void Save()
		{
			this._cfg.Save(this._strFileName);
			this._blnDirty = false;
		}
 
		public void Update(AppSetting NewSetting)
		{
			string text1 = "//add[@key='{0}']";
			XmlNode node1 = this._xAS.SelectSingleNode(string.Format(text1, NewSetting.Key));
			if (node1 == null)
			{
				throw new ArgumentOutOfRangeException("Key", NewSetting.Key, "The item you wanted to update does not exists.");
			}
			node1.Attributes.Item(1).Value = NewSetting.Value;
			this._blnDirty = true;
			if (this.AutoSave)
			{
				this.Save();
			}
		}
 




		
	}
}
